//
//  SJMapViewController.swift
//  GeliBusinessPlatform
//
//  Created by junye lu on 2020/5/6.
//  Copyright © 2020 junye lu. All rights reserved.
//

import UIKit
import AMapLocationKit
import AMapFoundationKit
import MAMapKit
import AMapSearchKit
import LGButton
import IQKeyboardManagerSwift
protocol SJMapViewControllerDelegate {
    func SJMapViewControllerSelectArd(adr:String)
}
class SJMapViewController: UIViewController,AMapLocationManagerDelegate,MAMapViewDelegate,AMapSearchDelegate,UISearchBarDelegate,SearchBarViewDelegate,UITableViewDelegate,UITableViewDataSource, CLLocationManagerDelegate{
    
    var delegate:SJMapViewControllerDelegate?
    
    var mapView: MAMapView!
    var search: AMapSearchAPI!
    var lat:CLLocationDegrees = 0
    var lng:CLLocationDegrees = 0
    var manger:CLLocationManager = CLLocationManager()
    
    var adrArr:Array<AMapPOI> = []
    
    
    var searchV : SearchBarView? = nil
    var listView:UITableView? = nil
    
    
    
    
    override func viewWillAppear(_ animated: Bool) {
        self.navigationController?.navigationBar.isHidden = true
        IQKeyboardManager.shared.shouldPlayInputClicks = true
        IQKeyboardManager.shared.enable = true
        IQKeyboardManager.shared.shouldResignOnTouchOutside = true
    }
    
    func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        manger.stopUpdatingLocation()
        let location = locations.last
        lat = location?.coordinate.latitude as! CLLocationDegrees
        lng = location?.coordinate.longitude as! CLLocationDegrees
        initMapView(lat: lat, lng: lng)
        initSearch()
        initUI()
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        AMapServices.shared().apiKey = "ca417b43e3d031db9c29382cc09a174a"
        
        manger.delegate = self
        manger.startUpdatingLocation()
        
       
    }
    
    //MARK: - 自定义delegate
    func clearSearchAction() {
        print("清楚搜索文字")
    }
    func searchBarSearchButtonClicked(_ searchBar: UISearchBar) {
        print("点击搜索进行--")
        searchBar.resignFirstResponder()
        let request = AMapPOIKeywordsSearchRequest()
        request.keywords = searchBar.text
        request.requireExtension = true
        request.cityLimit = true
        request.requireSubPOIs = true
        search.aMapPOIKeywordsSearch(request)
    }
    @objc func cancelKeyboard(){
        IQKeyboardManager.shared.resignFirstResponder()
    }
    @objc func backToCenter(){
        mapView.setCenter(CLLocationCoordinate2DMake(lat, lng), animated: false)
    }
    @objc func backAction(){
        self.navigationController?.popViewController(animated: true)
    }
    @objc func submitAction(){
        
        if adrStr != nil {
            delegate?.SJMapViewControllerSelectArd(adr: adrStr!)
            self.navigationController?.popViewController(animated: true)
        }else{
            HUD.flash(.label("请选择地址"), delay: 1.2)
        }
    }
    //MARK: - 高德地图delegate
    
    func onPOISearchDone(_ request: AMapPOISearchBaseRequest!, response: AMapPOISearchResponse!) {
        
        if response.count == 0 {
            return
        }
        //解析response获取POI信息，具体解析见 Demo
        adrArr.removeAll()
        response.pois.forEach { (poi) in
            adrArr.append(poi)
        }
        listView?.reloadData()
    }
    
    //移动后刷新
    func mapView(_ mapView: MAMapView!, mapDidMoveByUser wasUserAction: Bool) {
            let req = AMapPOIAroundSearchRequest()
            //地址
            req.location = AMapGeoPoint.location(withLatitude: CGFloat(mapView.centerCoordinate.latitude), longitude: CGFloat(mapView.centerCoordinate.longitude))
            req.requireExtension = true
            search.aMapPOIAroundSearch(req)
            idx = IndexPath(row: 0, section: 0)
            self.listView?.scrollRectToVisible(CGRect(x: 0, y: 0, width: 1, height: 1), animated: false)
    }
    
    var annotation:MAPointAnnotation? = nil
    func mapViewRegionChanged(_ mapView: MAMapView!) {
        
        let request = AMapReGeocodeSearchRequest()
        request.location = AMapGeoPoint.location(withLatitude: CGFloat(mapView.centerCoordinate.latitude), longitude: CGFloat(mapView.centerCoordinate.longitude))
        search.aMapReGoecodeSearch(request)
        request.requireExtension = true
        
        if annotation == nil
        {
            let pointAnnotation = MAPointAnnotation()
            annotation = pointAnnotation
            mapView.addAnnotation(annotation)
        }
        annotation!.coordinate = CLLocationCoordinate2D(latitude: mapView.centerCoordinate.latitude, longitude: mapView.centerCoordinate.longitude)
    }
    
    func mapView(_ mapView: MAMapView!, viewFor annotation: MAAnnotation!) -> MAAnnotationView! {
        if annotation.isKind(of: MAPointAnnotation.self) {
            let pointReuseIndetifier = "pointReuseIndetifier"
            var annotationView: MAAnnotationView? = mapView.dequeueReusableAnnotationView(withIdentifier: pointReuseIndetifier)
            
            if annotationView == nil {
                annotationView = MAAnnotationView(annotation: annotation, reuseIdentifier: pointReuseIndetifier)
            }
            
            annotationView!.image = UIImage(named: "dingzhen")
            //设置中心点偏移，使得标注底部中间点成为经纬度对应点
            annotationView!.centerOffset = CGPoint(x: 0, y: -15);
            return annotationView!
        }
        
        return nil
    }
    
    //MARK: - cell delegate
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return 73.5
    }
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return adrArr.count
    }
    var idx:IndexPath? = nil
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: "SJMapCell") as! SJMapCell
        
        let poi = adrArr[indexPath.row]
        cell.nameLbl.text = poi.name
        
        //1.将两个经纬度点转成投影点
        let point1 = MAMapPointForCoordinate(CLLocationCoordinate2D(latitude: lat, longitude: lng))
        let point2 = MAMapPointForCoordinate(CLLocationCoordinate2D(latitude: CLLocationDegrees(poi.location!.latitude), longitude: CLLocationDegrees(poi.location!.longitude)))
        
        //2.计算距离
        let distance = MAMetersBetweenMapPoints(point1,point2);
        
        cell.contentLbl.text = "\(Int(distance))米 | \(poi.address!)"
        cell.lat = poi.location.latitude
        cell.lng = poi.location.longitude
        cell.imgView.isHidden = true
        if indexPath == idx {
            cell.imgView.isHidden = false
            adrStr = poi.name
        }
        
        return cell
    }
    var adrStr:String?
    func tableView(_ tableView: UITableView, didSelectRowAt indexPath: IndexPath) {
        if idx == indexPath {
            return
        }else{
            idx = indexPath
            let cell = tableView.cellForRow(at: idx!) as! SJMapCell
            mapView.setCenter(CLLocationCoordinate2D(latitude:CLLocationDegrees(cell.lat!), longitude: CLLocationDegrees(cell.lng!)), animated: false)
            listView?.reloadData()
            adrStr = cell.nameLbl.text
        }
        
    }
    
    //MARK: - initUI
    func initUI(){
        let img = UIImageView()
        img.image = UIImage(named: "beijing")
        view.addSubview(img)
        img.snp.makeConstraints { (make) in
            make.top.right.left.equalTo(0)
            make.height.equalTo(200)
        }
        
        let btn = LGButton()
        btn.titleString = "确定"
        btn.titleColor = UIColor.white
        btn.titleFontSize = 13
        btn.gradientStartColor = UIColor(named: "按钮渐变色上")
        btn.gradientEndColor = UIColor(named: "按钮渐变色下,字体颜色")
        btn.gradientHorizontal = true
        btn.cornerRadius = 3
        btn.addTarget(self, action: #selector(submitAction), for: .touchUpInside)
        self.view.addSubview(btn)
        btn.snp.makeConstraints { (make) in
            make.top.equalTo(60)
            make.right.equalTo(-20)
            make.width.equalTo(60)
            make.height.equalTo(30)
        }
        
        let btn2 = UIButton()
        btn2.setTitle("取消", for: .normal)
        btn2.backgroundColor = UIColor(named: "灰色分界线")
        btn2.setTitleColor(UIColor(named: "个人中心灰色字体"), for: .normal)
        btn2.titleLabel?.font = UIFont.systemFont(ofSize: 13)
        btn2.addTarget(self, action: #selector(backAction), for: .touchUpInside)
        self.view.addSubview(btn2)
        btn2.snp.makeConstraints { (make) in
            make.top.equalTo(60)
            make.left.equalTo(20)
            make.width.equalTo(60)
            make.height.equalTo(30)
        }
    }
    func initBtmView(view:UIView){
        let searchBGView = UIView()
        searchBGView.backgroundColor = UIColor.white
        view.addSubview(searchBGView)
        searchBGView.snp.makeConstraints { (make) in
            make.top.left.right.equalTo(0)
            make.height.equalTo(44)
        }
        
        let tbv = UITableView()
        tbv.separatorStyle = .none
        tbv.delegate = self
        tbv.dataSource = self
        view.addSubview(tbv)
        listView = tbv
        tbv.snp.makeConstraints { (make) in
            make.top.equalTo(searchBGView.snp_bottom)
            make.bottom.left.right.equalTo(0)
        }
        listView?.register(UINib(nibName: "SJMapCell", bundle: nil), forCellReuseIdentifier: "SJMapCell")
        
        let searchBr = SearchBarView.init()
        searchBGView.addSubview(searchBr)
        searchBr.snp.makeConstraints { (make) in
            make.left.equalTo(10)
            make.right.equalTo(-10)
            make.centerY.equalToSuperview()
            make.height.equalTo(30)
        }
        searchV = searchBr
        searchBr.placeholderStr = "搜索地点"
        searchBr.cornerRadius = 35/2.0
        searchBr.delegate = self
        searchBr.delegateL = self
    }
    
    func initMapView(lat:CLLocationDegrees,lng:CLLocationDegrees) {
        let btmView = UIView()
        self.view.addSubview(btmView)
        btmView.snp.makeConstraints { (make) in
            make.bottom.right.left.equalTo(0)
            make.height.equalTo(260)
        }
        initBtmView(view: btmView)
        
        AMapServices.shared().enableHTTPS = true
        mapView = MAMapView()
        mapView.delegate = self
        mapView.zoomLevel = 16
        mapView.isZoomEnabled = true
        mapView.setCenter(CLLocationCoordinate2D(latitude: lat, longitude: lng), animated: false)
        self.view.addSubview(mapView!)
        mapView.snp.makeConstraints { (make) in
            make.top.left.right.equalTo(0)
            make.bottom.equalTo(btmView.snp_top)
        }
        let tap = UITapGestureRecognizer(target: self, action: #selector(cancelKeyboard))
        mapView.addGestureRecognizer(tap)
        
        let btn = UIButton()
        btn.setImage(UIImage(named: "dingwei"), for: .normal)
        btn.addTarget(self, action: #selector(backToCenter), for: .touchUpInside)
        view.addSubview(btn)
        btn.snp.makeConstraints { (make) in
            make.width.height.equalTo(48)
            make.right.equalTo(-17.5)
            make.bottom.equalTo(btmView.snp_top).offset(-13.5)
        }
    }
    
    func initSearch() {
        search = AMapSearchAPI()
        search.delegate = self
        let request = AMapPOIAroundSearchRequest()
        //地址
        request.location = AMapGeoPoint.location(withLatitude: CGFloat(mapView.centerCoordinate.latitude), longitude: CGFloat(mapView.centerCoordinate.longitude))
        request.requireExtension = true
        search.aMapPOIAroundSearch(request)
    }
}
